from unittest import TestCase
from slerp import *

log = logging.getLogger(__name__)


class TestValidator(TestCase):
	service = None
	
	def setUp(self):
		self.service = Service()
	
	def test_keys_fail(self):
		try:
			self.service.key_service({'user': {'name':'hell', 'alamat':'Jakarta'}})
		except ValidationException as e:
			self.assertEqual(e.key, 'nama')
			self.assertEqual(e.message, 'required.key')
	
	def test_keys_success(self):
		domain = self.service.key_service({'user': [{'nama': 'hell', 'alamat':''}, {'nama': 'hell', 'alamat':''}]})
		self.assertTrue('user' in domain)
		for data in domain['user']:
			self.assertTrue('nama' in data)
		
	def test_empty(self):
		try:
			self.service.value_service({'user': {'nama': ''}})
		except ValidationException as e:
			self.assertEqual(e.message, 'value.cannot.be.empty')
	
	def test_empty_success(self):
		domain = self.service.value_service({'user': {'nama': 'Rifky'}})
		self.assertTrue('user' in domain)
		self.assertTrue('nama' in domain['user'])
	
	def test_number_fail(self):
		try:
			self.service.number_service({'trx': {'trx_id': 'data'}})
		except ValidationException as e:
			self.assertEqual(e.message, 'required.number.trx_id')
			self.assertEqual(e.key, 'trx_id')
			
	def test_mail_fail(self):
		try:
			self.service.mail_service({'user': {'email': 'data'}})
		except ValidationException as e:
			self.assertEqual(e.message, 'invalid.email')
			self.assertEqual(e.key, 'email')

	def test_phone_fail(self):
		try:
			self.service.phone_service({'user': {'name': 'kiditz', 'phone': 'sa'}, 'alamat':'as'})
		except ValidationException as e:
			self.assertEqual(e.message, 'invalid.phone.number')


class Service(object):
	
	@Key(['user.nama.alamat'])
	def key_service(self, domain):
		return domain
	
	@Email(['user.email'])
	def mail_service(self, domain):
		return domain
	
	@Empty(['user.nama'])
	def value_service(self, domain):
		return domain
	
	@Number(['trx.trx_id'])
	def number_service(self, domain):
		return domain
	
	@Blank(['user.name.phone', 'alamat'])
	@Phone(['user.phone'])
	def phone_service(self, domain):
		return domain


import logging
import sys
from logging.config import dictConfig

syslog_format = ("[%(name)s][env:{logging_env}] %(levelname)s "
                 "[{hostname}  %(process)d] [%(filename)s:%(lineno)d] "
                 "- %(message)s").format(
    logging_env="", hostname="")

logging.basicConfig(filename='slerpio.log', level=logging.INFO)

logging.getLogger('sqlalchemy.engine').setLevel(logging.ERROR)
logging.getLogger('werkzeug').setLevel(logging.DEBUG)
logging.getLogger('socketIO-client').setLevel(logging.ERROR)
logging.getLogger('apscheduler.executors.default').setLevel(logging.ERROR)

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'standard': {
            'format': '%(asctime)s %(levelname)s %(process)d '
                      '[%(name)s] %(filename)s:%(lineno)d - %(message)s',
            },
        'syslog_format': {'format': syslog_format},
        'raw': {'format': '%(message)s'},
        },
    'handlers': {
        'console': {
            'level': 'INFO',
            'class': 'logging.StreamHandler',
            'formatter': 'standard',
            'stream': sys.stdout,
            }
    },
    'loggers': {
        'requests.packages.urllib3.connectionpool': {
            'handlers': [],
            'propagate': False,
            'level': 'INFO',
            },
        '': {
            'handlers': ['console'],
            'level': 'INFO',
            'propagate': False
        },
        }
}

dictConfig(LOGGING)


from unittest import TestCase

from slerp import (is_json, is_empty, is_blank, is_email, is_not_empty, is_palindrome, get_name_abbreviations, \
	random_colors)


class TestString(TestCase):
	
	def setUp(self):
		pass
	
	def is_json(self):
		self.assertEqual(is_json('{"key":"value"}'), True)
		self.assertEqual(is_json('ERROR'), False)
		self.assertEqual(is_json(False), False)

	def test_email(self):
		self.assertEqual(is_email('kiditzbastara@gmail.com'), True)
		self.assertEqual(is_email('kiditzbastaragmail.com'), False)
		self.assertEqual(is_email('kiditzbastara@gmail'), False)

	def test_blank(self):
		self.assertEqual(is_blank('        '), True)
		self.assertEqual(is_blank(' '), True)
		self.assertEqual(is_blank(''), True)
		self.assertEqual(is_blank('TEST'), False)

	def test_empty(self):
		self.assertEqual(is_empty(''), True)
		self.assertEqual(is_empty(' '), False)
		self.assertEqual(is_empty('empty'), False)

	def test_not_empty(self):
		self.assertEqual(is_not_empty('  '), True)
		self.assertEqual(is_not_empty(''), False)
		self.assertEqual(is_not_empty('empty'), True)

	def test_palindrome(self):
		self.assertEqual(is_palindrome('TAMAT'), True)
		self.assertEqual(is_palindrome('Tamat'), False)
		self.assertEqual(is_palindrome('Pisang'), False)

	def test_random_color(self):
		color = random_colors(use_hastag=True)
		self.assertEqual(color.startswith('#'), True)
		color = random_colors()
		self.assertEqual(color.startswith('#'), False)

	def test_get_simple_name(self):
		self.assertEqual(get_name_abbreviations('rifky aditya bastara'), 'RAB')
		self.assertEqual(get_name_abbreviations('hidayanti anatya bastari jelek'), 'HAB')
		self.assertEqual(get_name_abbreviations('hidayanti anatya'), 'HA')
		self.assertEqual(get_name_abbreviations('hidayanti anatya bastari jelek banget'), 'HAB')


from kafka import KafkaProducer
from app import app
from app import log


def send_message(topic, domain, partition=0):
	producer = KafkaProducer(bootstrap_servers=app.config['KAFKA_SERVER'], value_serializer=lambda v: json.dumps(v).encode('UTF-8'))
	future = producer.send(topic, domain, partition=partition)
	try:
		future.get(timeout=app.config['KAFKA_TIME_OUT'])
	except Exception as e:
		log.error(e, exc_info=True)


import json

from flask import Flask
from flask_sqlalchemy import SQLAlchemy

from consul import Consulate
from response import JsonResponse
from exception import ValidationException, CoreException
from logger import logging
import os

log = logging.getLogger(__name__)

DEFAULT_CONFIG = 'config.json'
if not os.path.exists(DEFAULT_CONFIG):
	raise RuntimeError('please create config.json file in your project root folder')


def init_apps(configuration):
	apps = Flask(__name__)
	consul = Consulate(app=apps)
	consul.load_config(namespace=configuration['namespace'])
	consul.register(name=configuration['name'], port=configuration['port'], interval=configuration['interval'], httpcheck=configuration['health'])
	return apps


json_file = open(DEFAULT_CONFIG, 'rb')
config = json.load(json_file)
json_file.close()
app = init_apps(config)
db = SQLAlchemy(app=app)
app.response_class = JsonResponse
log.info('consul-config : %s', app.config)
log.info('config : %s', config)


@app.route('/test')
def index():
	raise ValidationException(message='Data tidak boleh kosong', key='data.cannot.nill')
	

@app.errorhandler(Exception)
# noinspection PyTypeChecker
def handle_error(error):
	msg = {'status': 'FAIL'}
	if type(error) is TypeError:
		msg.update({'message': 'type.error'})
	elif type(error) is ValidationException:
		msg.update({'message': error.message})
		if error.key is not None:
			msg.update({'key': error.key})
			pass
	elif type(error) is CoreException:
		msg.update({'message': error.message})
	else:
		message = [str(x) for x in error.args]
		msg.update({'message': message})
		
	log.error(error, exc_info=True)
	return msg


def run(**kwargs):
	kwargs.update({'port': app.config['SERVER_PORT']})
	if 'SERVER_HOST' in app.config:
		kwargs.update({'host': app.config['SERVER_HOST']})
	app.run(**kwargs)